.TH "Simple Virtual Machine - Plugin tribe" 7 "2023-06-20"
.SH SYNOPSIS
.B svm_plugin_tribe
An easy mechanism to let SVM processes exchange values.
.SH DESCRIPTION
This plugin creates the concept of tribe of processes.
A process is part of the tribe when attached to the tribe scheduler.
.P
When a process is part of the tribe, it can receive values from other processes within the tribe, and send values to another process just by specifying its name as a target.
.SH INVOKATION
This plugin can be added to the Simple Virtual Machine using one of these directives:
.SS Local
.nf

LOCAL PLUGIN "svmplugintribe/libsvmtribe.so"

.fi
when compiled/installed locally and should be accessible from the application directory.
.SS Machine and System
.nf

PLUGIN "===PLUGINLIB==="

.fi
when installed with auto-tools and should be accessible from the Simple Virtual Machine installation directory or from system directories. (Recommended)
.SS Absolute path
.nf

PLUGIN "===PLUGINLIBDIR===/===PLUGINLIB==="

.fi
when installed with auto-tools and should be accessible from an absolute path.
.SH CONTENT
.SS INTERRUPTION tribe.wrong_source
This interruption is raised when values are sent from a process not being in the tribe.
.SS INTERRUPTION tribe.wrong_destination
This interruption is raised when values are sent to a process not being in the tribe.
.SS INTERRUPTION tribe.interrupted
This interruption is raised when a process within the tribe is waiting for values, and has been notified to exit before receiving values.
.SS STRUCT tribe.identity
This structure is used to convey the name of the source process when values are received by a process within the tribe.
.SS STRUCT tribe.message
This structure contains the values to be transmitted between tribe processes.
.SS TYPE tribe.process
This type fakes a process having an address within the tribe.
It can be used by processes external to the tribe to exchange data with processes within the tribe.
.P
Usage of this type should be limited to corner cases.
It is usually better to have processes attached to the tribe scheduler.
.SS WAITING INSTRUCTION tribe.process -> tribe.process
This instruction artificially adds the current process to the tribe without attaching it to the tribe scheduler.
.P
This lasts until the value is deleted.
.SS WAITING INSTRUCTION tribe.send STR:destination_name PTR:values
This instruction sends values referenced by the pointer to the tribe member having the destination_name.
.I
The instruction clears the values sent, and there are no longer accessible from the sender.
.P
The !tribe.wrong_source interruption is raised when the current process is not part of the tribe.
.P
The !tribe.wrong_destination interruption is raised when the destination process is not part of the tribe.
.P
There is no guarranted delivery of the values to the destination tribe member.
.SS WAITING INSTRUCTION tribe.recv MUTABLE STR:source_name PTR:emplace ? -> PTR:values
This instruction waits for values from another tribe member.
.P
The name of the tribe member sending the values is written into the source_name parameter.
.P
When the emplace pointer is specified, values are written to this pointer or a MEMORY interruption is raised if the values can not be written in an atomic way.
Otherwise, the memory is allocated.
The pointer to the received values is returned.
.P
The !tribe.wrong_destination interruption is raised when the current process is not part of the tribe.
.P
If the instruction gets a notification before receiving values, a !tribe.interrupted interruption is raised.
.SS SCHEDULER tribe.scheduler
This is the tribe scheduler.
.P
When a process is attached to it, this process becomes a member of the tribe and can start to exchange values with other tribe members.
.P
The scheduler will run every process attached to it like the run.parallel does.
.SH EXAMPLES
.SS Simple exchange
.nf
#!/usr/bin/env svm
LOG
PLUGIN "svmcom.so"
PLUGIN "===PLUGINLIB==="

PROCESS "a"
	CODE "main" INLINE
		:debug BREAK "a"
		:memory STR/s, (INT, STR)/p
		[ 17 , "lol" ] -> p
		:tribe.send "b" p
		"" -> &s
		:tribe.recv @&s p
		:com.message @(p/0) " " @(p/1)
	END
	SCHEDULER tribe.scheduler
END

PROCESS "b"
	CODE "main" INLINE
		:debug BREAK "b"
		:memory STR/s, (INT, STR)/p
		"" -> &s
		:tribe.recv @&s p
		:com.message @(p/0) " " @(p/1)
		:shift (p/0)
		"mdr" -> (p/1)
		:tribe.send @&s p
	END
	SCHEDULER tribe.scheduler
END
.fi
.SS Complete example
.nf
#!/usr/bin/env svm
LOG
PLUGIN "svmstr.so"
PLUGIN "svmcom.so"
PLUGIN "===PLUGINLIB==="
ARGUMENT STR port
PROCESS "wait"
	CODE "main" INLINE
		:interruption GLOBAL TERMINATE end
		:interruption !com.interrupted ignore
		:memory com.device/server, com.device/client
		:com.open com.tcp < "0.0.0.0" @&port -> &server
	:label loop
		:com.command @&server CLIENT -> &client
		:tribe.send "recv" client
		:goto loop
	:label ignore
		:return
	:label end
	END
	MEMORY port
	SCHEDULER tribe.scheduler
END
PROCESS "recv"
	CODE "main" INLINE
		:interruption !tribe.interrupted end
		:memory STR/s, (com.device, STR)/m
	:label loop
		"" -> &s
		:tribe.recv @&s &m*1
		:com.read @(m/0) com.line -> (m/1)
		:tribe.send "compute" m
		:goto loop
	:label end
	END
	SCHEDULER tribe.scheduler
END
PROCESS "compute"
	CODE "main" INLINE
		:interruption !tribe.interrupted end
		:memory STR/s, (com.device, STR)/m
	:label loop
		"" -> &s
		:tribe.recv @&s m
		:str.replace @(m/1) 1 CONST str.pattern "\n" => ""
		:str.join "{" @(m/1) "}\n" -> (m/1)
		:tribe.send "send" m
		:goto loop
	:label end
	END
	SCHEDULER tribe.scheduler
END
PROCESS "send"
	CODE "main" INLINE
		:interruption !tribe.interrupted end
		:memory STR/s, (com.device, STR)/m
	:label loop
		"" -> &s
		:tribe.recv @&s m
		:com.write @(m/0) @(m/1)
		[ , ] -> m
		:goto loop
	:label end
	END
	SCHEDULER tribe.scheduler
END
.fi
.SH SEE ALSO
.BR svm (1)
for help on the Simple Virtual Machine.
.SH AUTHOR
This plugin has been written by Julien BRUGUIER and is maintained by Julien BRUGUIER <projet.svm@pappy.tf>.
